package zero1hd.rhythmbullet.audio;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.channels.Channels;
import java.security.InvalidParameterException;

import javax.sound.sampled.AudioFormat;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.audio.Music.OnCompletionListener;
import com.badlogic.gdx.files.FileHandle;

import zero1hd.rhythmbullet.audio.wavedecoder.WavDecoder;

public class WAVManager implements MusicManager {
	private int readWindowSize = 1024;
	private AudioFormat format;
	private int playbackIndex;
	private int readWindowIndex;
	private Music playbackMusic;
	WavDecoder decoder;
	private FileHandle file;
	private String basicSongName;
	private RandomAccessFile raf;
	
	public WAVManager(FileHandle file) {
		this.file = file;
		basicSongName = file.name();
		try {
			raf = new RandomAccessFile(file.file(), "r");
			decoder = new WavDecoder(new BufferedInputStream(Channels.newInputStream(raf.getChannel())));
		} catch (InvalidParameterException | IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		playbackMusic = Gdx.audio.newMusic(file);
		format = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, (float) decoder.getSampleRate(), 16, decoder.getChannels(), decoder.getChannels()*2, (float) decoder.getSampleRate(), false);
	}
	
	@Override
	public void playbackIndexUpdate() {
		playbackIndex = (int) (playbackMusic.getPosition() * decoder.getSampleRate() / readWindowSize);
	}
	
	@Override
	public int getPlaybackIndexPosition() {
		return playbackIndex;
	}
	
	@Override
	public int getReadWindowSize() {
		return readWindowSize;
	}
	
	@Override
	public void setReadIndexToPlaybackIndex() {
		try {
			long pointer = (long) (getPositionInSeconds()*(long)getSampleRate())-readWindowSize;
			raf.seek(pointer);
			readWindowIndex = (int) (pointer/readWindowSize);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	@Override
	public int readSamples(float[] samples) {
		int samplesRead = 0;
		try {
			samplesRead = decoder.readSamples(samples);
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		if (samplesRead != 0) {
			readWindowIndex++;
		}
		return samplesRead;
	}

	@Override
	public long getSampleCount() {
		return decoder.getFrameLength();
	}

	@Override
	public void dispose() {
		playbackMusic.stop();
		playbackMusic.dispose();
		decoder.cleanAndClose();
	}

	@Override
	public float getDuration() {
		return decoder.getDurationInSeconds();
	}

	@Override
	public float getSampleRate() {
		return format.getSampleRate();
	}

	@Override
	public int getreadWindowIndex() {
		return readWindowIndex;
	}

	@Override
	public void pause() {
		playbackMusic.pause();
	}

	@Override
	public void play() {
		playbackMusic.play();
	}

	@Override
	public boolean isPlaying() {
		return playbackMusic.isPlaying();
	}

	@Override
	public float getPositionInSeconds() {
		return playbackMusic.getPosition();
	}

	@Override
	public void setPosition(float position) {
		playbackMusic.setPosition(position);
	}

	@Override
	public void setOnCompletionListener(OnCompletionListener listener) {
		playbackMusic.setOnCompletionListener(listener);
	}

	@Override
	public void setVolume(float percent) {
		playbackMusic.setVolume(percent);
	}

	@Override
	public boolean isFinishedLoading() {
		return true;
	}
	
	@Override
	public String getBasicSongName() {
		return basicSongName;
	}

	@Override
	public FileHandle getMusicFile() {
		return file;
	}
}
